@@ -1,7 +1,7 @@
 /*
  * Copyright © 2009      CNRS
- * Copyright © 2009-2011 INRIA.  All rights reserved.
- * Copyright © 2009-2011 Université Bordeaux 1
+ * Copyright © 2009-2011 inria.  All rights reserved.
+ * Copyright © 2009-2012 Université Bordeaux 1
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  *
  * See COPYING in top-level directory.
@@ -20,7 +20,9 @@
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
-
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
 #include <string.h>
 
 #ifdef HWLOC_HAVE_ATTRIBUTE_FORMAT
@@ -45,9 +47,10 @@ typedef enum hwloc_backend_e {
   HWLOC_BACKEND_NONE,
   HWLOC_BACKEND_SYNTHETIC,
 #ifdef HWLOC_LINUX_SYS
-  HWLOC_BACKEND_SYSFS,
+  HWLOC_BACKEND_LINUXFS,
 #endif
   HWLOC_BACKEND_XML,
+  HWLOC_BACKEND_CUSTOM,
   /* This value is only here so that we can end the enum list without
      a comma (thereby preventing compiler warnings) */
   HWLOC_BACKEND_MAX
@@ -108,6 +111,7 @@ struct hwloc_topology {
   struct hwloc_topology_support support;
 
   struct hwloc_os_distances_s {
+    hwloc_obj_type_t type;
     int nbobjs;
     unsigned *indexes; /* array of OS indexes before we can convert them into objs. always available.
 			*/
@@ -120,16 +124,19 @@ struct hwloc_topology {
 		       * will be copied into the main logical-index-ordered distance at the end of the discovery.
 		       */
     int forced; /* set if the user forced a matrix to ignore the OS one */
-  } os_distances[HWLOC_OBJ_TYPE_MAX];
+
+    struct hwloc_os_distances_s *prev, *next;
+  } *first_osdist, *last_osdist;
 
   hwloc_backend_t backend_type;
   union hwloc_backend_params_u {
 #ifdef HWLOC_LINUX_SYS
-    struct hwloc_backend_params_sysfs_s {
-      /* sysfs backend parameters */
+    struct hwloc_backend_params_linuxfs_s {
+      /* FS root parameters */
       char *root_path; /* The path of the file system root, used when browsing, e.g., Linux' sysfs and procfs. */
       int root_fd; /* The file descriptor for the file system root, used when browsing, e.g., Linux' sysfs and procfs. */
-    } sysfs;
+      struct utsname utsname; /* cached result of uname, used multiple times */
+    } linuxfs;
 #endif /* HWLOC_LINUX_SYS */
 #if defined(HWLOC_OSF_SYS) || defined(HWLOC_COMPILE_PORTS)
     struct hwloc_backend_params_osf {
@@ -142,9 +149,15 @@ struct hwloc_topology {
       void *doc;
 #endif /* HWLOC_HAVE_LIBXML2 */
       char *buffer; /* only used when not using libxml2 */
+      struct hwloc_xml_imported_distances_s {
+	hwloc_obj_t root;
+	struct hwloc_distances_s distances;
+	struct hwloc_xml_imported_distances_s *prev, *next;
+      } *first_distances, *last_distances;
     } xml;
     struct hwloc_backend_params_synthetic_s {
       /* synthetic backend parameters */
+      char *string;
 #define HWLOC_SYNTHETIC_MAX_DEPTH 128
       unsigned arity[HWLOC_SYNTHETIC_MAX_DEPTH];
       hwloc_obj_type_t type[HWLOC_SYNTHETIC_MAX_DEPTH];
@@ -159,16 +172,18 @@ extern void hwloc_setup_pu_level(struct hwloc_topology *topology, unsigned nb_pu
 extern int hwloc_get_sysctlbyname(const char *name, int64_t *n);
 extern int hwloc_get_sysctl(int name[], unsigned namelen, int *n);
 extern unsigned hwloc_fallback_nbprocessors(struct hwloc_topology *topology);
+extern void hwloc_connect_children(hwloc_obj_t obj);
+extern int hwloc_connect_levels(hwloc_topology_t topology);
+
 
 #if defined(HWLOC_LINUX_SYS)
-extern void hwloc_look_linux(struct hwloc_topology *topology);
-extern void hwloc_set_linux_hooks(struct hwloc_topology *topology);
-extern int hwloc_backend_sysfs_init(struct hwloc_topology *topology, const char *fsroot_path);
-extern void hwloc_backend_sysfs_exit(struct hwloc_topology *topology);
+extern void hwloc_look_linuxfs(struct hwloc_topology *topology);
+extern void hwloc_set_linuxfs_hooks(struct hwloc_topology *topology);
+extern int hwloc_backend_linuxfs_init(struct hwloc_topology *topology, const char *fsroot_path);
+extern void hwloc_backend_linuxfs_exit(struct hwloc_topology *topology);
 #endif /* HWLOC_LINUX_SYS */
 
 extern int hwloc_backend_xml_init(struct hwloc_topology *topology, const char *xmlpath, const char *xmlbuffer, int buflen);
-extern void hwloc_xml_check_distances(struct hwloc_topology *topology);
 extern int hwloc_look_xml(struct hwloc_topology *topology);
 extern void hwloc_backend_xml_exit(struct hwloc_topology *topology);
 
@@ -260,6 +275,7 @@ extern void hwloc_insert_object_by_parent(struct hwloc_topology *topology, hwloc
 /* Insert uname-specific names/values in the object infos array */
 extern void hwloc_add_uname_info(struct hwloc_topology *topology);
 
+#ifdef HWLOC_INSIDE_LIBHWLOC
 /** \brief Return a locally-allocated stringified bitmap for printf-like calls. */
 static __hwloc_inline char *
 hwloc_bitmap_printf_value(hwloc_const_bitmap_t bitmap)
@@ -317,6 +333,7 @@ hwloc_setup_level(int procid_max, unsigned num, unsigned *osphysids, unsigned *p
     }
   hwloc_debug("%s", "\n");
 }
+#endif
 
 /* This can be used for the alloc field to get allocated data that can be freed by free() */
 void *hwloc_alloc_heap(hwloc_topology_t topology, size_t len);
@@ -340,15 +357,17 @@ hwloc_alloc_or_fail(hwloc_topology_t topology, size_t len, int flags)
   return hwloc_alloc(topology, len);
 }
 
-extern void hwloc_topology_distances_init(struct hwloc_topology *topology);
-extern void hwloc_topology_distances_clear(struct hwloc_topology *topology);
-extern void hwloc_topology_distances_destroy(struct hwloc_topology *topology);
-extern void hwloc_topology__set_distance_matrix(struct hwloc_topology *topology, hwloc_obj_type_t type, unsigned nbobjs, unsigned *indexes, hwloc_obj_t *objs, float *distances, int force);
-extern void hwloc_store_distances_from_env(struct hwloc_topology *topology);
-extern void hwloc_convert_distances_indexes_into_objects(struct hwloc_topology *topology);
-extern void hwloc_finalize_logical_distances(struct hwloc_topology *topology);
-extern void hwloc_restrict_distances(struct hwloc_topology *topology, unsigned long flags);
-extern void hwloc_free_logical_distances(struct hwloc_distances_s *dist);
+extern void hwloc_distances_init(struct hwloc_topology *topology);
+extern void hwloc_distances_clear(struct hwloc_topology *topology);
+extern void hwloc_distances_destroy(struct hwloc_topology *topology);
+extern void hwloc_distances_set(struct hwloc_topology *topology, hwloc_obj_type_t type, unsigned nbobjs, unsigned *indexes, hwloc_obj_t *objs, float *distances, int force);
+extern void hwloc_distances_set_from_env(struct hwloc_topology *topology);
+extern void hwloc_distances_restrict_os(struct hwloc_topology *topology);
+extern void hwloc_distances_restrict(struct hwloc_topology *topology, unsigned long flags);
+extern void hwloc_distances_finalize_os(struct hwloc_topology *topology);
+extern void hwloc_distances_finalize_logical(struct hwloc_topology *topology);
+extern void hwloc_clear_object_distances(struct hwloc_obj *obj);
+extern void hwloc_clear_object_distances_one(struct hwloc_distances_s *distances);
 extern void hwloc_group_by_distances(struct hwloc_topology *topology);
 
 #endif /* HWLOC_PRIVATE_H */
