@@ -513,11 +513,9 @@ hwloc_pci_find_hostbridge_parent(struct hwloc_topology *topology, struct hwloc_o
   /* restrict to the existing topology cpuset to avoid errors later */
   hwloc_bitmap_and(cpuset, cpuset, topology->levels[0][0]->cpuset);
 
-  /* why not inserting a group and let the core remove it if useless?
-   * 1) we need to make sure that the group is above all objects
-   * with same cpuset (to avoid attaching to caches or so)
-   * 2) the merge-keep-structure code is already done when coming here
-   */
+  /* if the remaining cpuset is empty, take the root */
+  if (hwloc_bitmap_iszero(cpuset))
+    hwloc_bitmap_copy(cpuset, hwloc_topology_get_topology_cpuset(topology));
 
   /* attach the hostbridge now that it contains the right objects */
   parent = hwloc_get_obj_covering_cpuset(topology, cpuset);
